#ifndef __MODPROBE_H
#define __MODPROBE_H

#ifndef MODULE_DIR
#define MODULE_DIR "/lib/modules"
#endif

#define PREFERRED_ALIASES "/lib/module-init-tools/ldetect-lst-modules.alias"

extern int use_binary_indexes;

/* structures */
/* keep track of module options from config file(s) */
struct module_options
{
	struct module_options *next;
	char *modulename;
	char *options;
};

/* keep track of module install commands from config file(s) */
struct module_command
{
	struct module_command *next;
	char *modulename;
	char *command;
};

/* keep track of module aliases added in the config file(s) */
struct module_alias
{
	struct module_alias *next;
	char *aliasname;
	char *module;
};

/* keep track of modules blacklisted in the config file(s) */
struct module_blacklist
{
	struct module_blacklist *next;
	char *modulename;
};

/* keep track of module softdeps added in the config file(s) */
struct module_softdep
{
	struct module_softdep *next;
	char *buf;
	/* The modname and string tables point to buf. */
	char *modname;
	struct string_table *pre;
	struct string_table *post;
};

/* keep track of all config options */
struct modprobe_conf
{
	struct module_options *options;
	struct module_command *commands;
	struct module_alias *aliases;
	struct module_blacklist *blacklist;
	struct module_softdep *softdeps;
};

struct module {
	struct list_head list;
	char *modname;
	char filename[0];
};

typedef enum
{
	mit_remove = 1,
	mit_dry_run = 2,
	mit_first_time = 4,
	mit_use_blacklist = 8,
	mit_ignore_commands = 16,
	mit_ignore_loaded = 32,
	mit_quiet_inuse = 64,
	mit_strip_vermagic = 128,
	mit_strip_modversion = 256,
	mit_resolve_alias = 512

} modprobe_flags_t;


/* functions' prototypes */

void
apply_blacklist(struct module_alias **,
		const struct module_blacklist *);

int read_config(const char *filename,
		const char *name,
		int dump_only,
		int removing,
		struct module_options **options,
		struct module_command **commands,
		struct module_alias **aliases,
		struct module_blacklist **blacklist);

void read_toplevel_config(const char *filename,
				 const char *name,
				 int dump_only,
				 int removing,
				 struct module_options **options,
				 struct module_command **commands,
				 struct module_alias **aliases,
				 struct module_blacklist **blacklist);

int read_config_file(const char *filename,
			    const char *name,
			    int dump_only,
			    int removing,
			    struct module_options **options,
			    struct module_command **commands,
			    struct module_alias **aliases,
			    struct module_blacklist **blacklist);

int read_config_file_bin(const char *filename,
			const char *name,
			int dump_only,
			int removing,
			struct module_options **options,
			struct module_command **commands,
			struct module_alias **aliases,
			struct module_blacklist **blacklist);

struct module_options *
add_options(const char *modname,
	    const char *option,
	    struct module_options *options);

char *strsep_skipspace(char **string, char *delim);
char *underscores(char *string);
struct module_alias *
add_alias(const char *, const char *, struct module_alias*);
char *getline_wrapped(FILE *file, unsigned int *linenum);
int
find_blacklist(const char *modname, const struct module_blacklist *blacklist);

struct module_blacklist *
add_blacklist(const char *modname, struct module_blacklist *blacklist);

const struct module_softdep *
find_softdep(const char *modname, const struct module_softdep *softdeps);

const char *find_command(const char *modname,
				const struct module_command *commands);

struct module_alias *find_matching_aliases(const char *modname,
		const char *cmdline_opts,
		const struct modprobe_conf *conf,
		const char *dirname,
		errfn_t error,
		modprobe_flags_t flags,
		struct list_head *list);

void read_depends(const char *dirname,
			 const char *start_name,
			 struct list_head *list);

void free_aliases(struct module_alias *alias_list);

int read_aliases(const char *filename,
			const char *name,
			int dump_only,
			struct module_alias **aliases);

int parse_config_file(const char *filename,
			    struct modprobe_conf *conf,
			    int dump_only,
			    int removing);

void parse_toplevel_config(const char *filename,
				  struct modprobe_conf *conf,
				  int dump_only,
				  int removing);

#endif /* __MODPROBE_H */
